# Calif v4.0
# This is a Shiny web app
# The calibration tool prepared by Statistical Office of the Slovak Republic

# Copyright (C) 2014-2021 The Statistical Office of the Slovak Republic

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Calif makes use of R packages shiny, sampling, nleqslv and haven.
# Small changes were made to the calib function of the sampling package and 
# to the pie function of the graphics package.

# Enter calif() in the command line
# Last update 22.4.2021

if (!require(shiny)) install.packages('shiny')
if (!require(sampling)) install.packages('sampling')
if (!require(nleqslv)) install.packages('nleqslv')
if (!require(haven)) install.packages('haven')
library(shiny)
library(sampling)
library(nleqslv)
library(haven)

message('To launch Calif run the calif() command')

calif <- function() {
options(shiny.maxRequestSize = Inf)

ui <- function(request)
{
  navbarPage(
    title = 'Calif 4.0',
    tabPanel(
      title = 'Overview',
      tags$div(style = 'text-align: center; font-size: 120%', 
               tags$h2('Welcome to Calif!'), 
               tags$p('Calif is the Shiny web application used for calibration of weights in statistical surveys. 
                      It is an open-source software and you are free to use it, also without any knowledge of R programming language.'),
               tags$p('Detailed information on how to use Calif can be found in the', 
                      tags$a(href = 'https://slovak.statistics.sk/wps/wcm/connect/8a164814-ebda-43c1-b88e-28d40cb63a77/Calif_Manual_v4.0.pdf?MOD=AJPERES&CVID=m6IKz0d&CVID=m6IKz0d&CVID=m6IKz0d',
                        'Manual', class = "btn btn-primary btn-xs")),
               tags$hr(),
               tags$h3('Getting started'),
               tags$p('To get started working with Calif, you first need to prepare the data and the table of totals (which are the known population marginals). 
                      You can learn it in the Manual, in fact it is very easy and intuitive.'), 
               tags$p('Once they are ready, you need to load them into Calif in the', actionButton('next_tab', 'next tab', class = "btn btn-primary btn-xs"), 
                      ' After that, you select the calibration variables, either categorical or numerical, initial weights for calibration, 
                      possible stratification and calibration methods.'),
               tags$hr(),
               tags$h3('Optimal strategy'),
               tags$p('Then you can calibrate the weights. In Calif, after each calibration step, you are provided with useful statistics that can help you 
                      to decide which method and parameters are most suitable for performing the best possible calibration.'),
               tags$p('The optimal strategy is to find the new weights that reproduce the population totals and are as close as possible to the initial 
                      weights such that the lower and upper bounds form a narrow interval (optimally close to 1 from both sides)'),
               tags$p('and the average difference between the initial and calibration weights is low in comparison to the linear solution. 
                      The feasibility of the average difference is demonstrated by a pie chart on the', tags$b('Calibration'), 'tab.'),
               tags$p('If you set too tense bounds, even if the solution is found, the histogram of quotients on the', 
                      tags$b('Calibration'), 'tab will look unnaturally, the average difference will be high and such calibration will not be appropriate.'),
               tags$hr(),
               tags$h3('Working directory'),
               tags$p('Your actual working directory is set to', tags$code(textOutput('my_wd', inline = TRUE)), '. If you want to change it, 
                      you can do it by clicking on this', actionButton('change_wd', 'button', class = "btn btn-primary btn-xs"), 'or later when saving the outputs.'),
               tags$hr(),
               tags$h3('Bookmarking'),
               tags$p('You can bookmark your work at any time by clicking on the', tags$b('Bookmark'), 'button on the', tags$b('Calibration'), 'tab. 
                      The files and settings are saved into your working directory and you obtain a link that can be used to reproduce your work when Calif is run again.'),
               tags$hr(),
               tags$h3('Output'),
               tags$p('You can save your work at any time by clicking on the', tags$b('Save'), 'button on the', tags$b('Calibration'), 'tab. 
                      The output is an identical copy of your data with column', tags$em('CalibrationWeight'), 'added. 
                      This column represents your latest calibration attempt, i.e.'),
               tags$p('if you tried several calibrations with different settings, the last one is recorded. 
                      If you haven\'t performed any calibration yet,', tags$em('CalibrationWeight'), 'column contains the initial weights.'),
               tags$p('If you carry out stratified calibration,', tags$em('CalibrationWeight'), 'column contains the new weights for 
                      strata that have been already calibrated and the initial weights for strata that have remained untouched.'),
               tags$p('Although the latest state of play is still remembered throughout all strata, you are recommended to regularly save or bookmark your work in order not 
                      to lose it if some unexpected error occurs.')
               )
               ),
    tabPanel(
      title = 'Data',
      sidebarLayout(
        sidebarPanel(width = 3,
                     conditionalPanel(
                       condition = 'input.switch_to_two == input.switch_to_one',
                       actionButton('switch_to_two', 'Switch to two-stage calibration', class = "btn btn-primary btn-block", icon = icon('refresh', lib = 'glyphicon')),
                       tags$h4('Load data', style = 'color: #367AB4'),
                       fileInput('load_data', label = NULL, accept = c('.txt', '.csv', '.sas7bdat'))
                     ),
                     conditionalPanel(
                       condition = 'input.switch_to_two > input.switch_to_one',
                       actionButton('switch_to_one', 'Switch to one-stage calibration', class = "btn btn-primary btn-block", icon = icon('refresh', lib = 'glyphicon')),
                       tags$h4('Load data - Household file', style = 'color: #367AB4'),
                       fileInput('load_data_HH', label = NULL, accept = c('.txt', '.csv', '.sas7bdat'))
                     ),
                     radioButtons('separator1', 'Separator', c('Semicolon' = ';', 'Comma' = ',', 'Space' = ' ', 'Tab' = '\t'), inline = TRUE),
                     radioButtons('decimal1', 'Decimal', c('Period' = '.', 'Comma' = ','), inline = TRUE),
                     tags$hr(),
                     conditionalPanel(
                       condition = 'input.switch_to_two > input.switch_to_one',
                       tags$h4('Load data - Individual file', style = 'color: #367AB4'),
                       fileInput('load_data_I', label = NULL, accept = c('.txt', '.csv', '.sas7bdat')),
                       radioButtons('separator3', 'Separator', c('Semicolon' = ';', 'Comma' = ',', 'Space' = ' ', 'Tab' = '\t'), inline = TRUE),
                       radioButtons('decimal3', 'Decimal', c('Period' = '.', 'Comma' = ','), inline = TRUE),
                       tags$hr()
                     ),
                     tags$h4('Load totals', style = 'color: #367AB4'),
                     fileInput('load_totals', label = NULL, accept = c('.txt', '.csv', '.sas7bdat')),
                     radioButtons('separator2', 'Separator', c('Semicolon' = ';', 'Comma' = ',', 'Space' = ' ', 'Tab' = '\t'), inline = TRUE),
                     radioButtons('decimal2', 'Decimal', c('Period' = '.', 'Comma' = ','), inline = TRUE),
                     tags$hr(),
                     conditionalPanel(
                       condition = 'input.switch_to_two > input.switch_to_one',
                       fluidRow(
                         column(6,
                                selectInput('ID_HH', 'Household ID - HH file', NULL)),
                         column(6,
                                selectInput('ID_I', 'Household ID - Ind. file', NULL))
                       ),
                       tags$hr()
                     ),
                     tags$h4('Specify calibration variables', style = 'color: #367AB4'),
                     checkboxInput('stratification', 'Stratification'),
                     fluidRow(
                       column(6,
                              selectInput('weights', 'Weights', NULL)
                       ),
                       column(6,
                              conditionalPanel(
                                condition = 'input.stratification == true',
                                selectInput('stra', 'Stratification variable', NULL)
                              )
                       )
                     ),
                     fluidRow(
                       column(6,
                              selectInput('num', 'Numerical variables', NULL, multiple = TRUE, selectize = FALSE, size = 8)),
                       column(6,
                              selectInput('cat', 'Categorical variables', NULL, multiple = TRUE, selectize = FALSE, size = 8))
                     ),
                     tags$h4('Other settings', style = 'color: #367AB4'),
                     fluidRow(
                       column(6,
                              selectInput('indicators', 'Indicators monitoring (optional)', NULL, multiple = TRUE, selectize = FALSE, size = 5)),
                       column(6,
                              tags$br(),
                              tags$br(),
                              checkboxGroupInput('indicators_stats', NULL, choices = c('Total (%)', 'Total (value)', 'Mean (%)', 'Mean (value)')))
                     ),
                     textInput('miss', 'Missings', placeholder = 'Missings for categorical, separated with commas'),
                     checkboxInput('eliminate_miss', 'Eliminate rows with missings'),
                     numericInput('maxit', 'Maximum nr of iterations', 20000, step = 1000, width = '45%'),
                     numericInput('tol', 'Tolerance', 0.00001, step = 0.000001, width = '45%'),
                     fluidRow(
                       actionButton('proceed', 'Proceed', class = "btn btn-primary btn-block")
                     )
        ),
        mainPanel(width = 9,
                  conditionalPanel(
                    condition = 'input.switch_to_two == input.switch_to_one',
                    tags$h3('Loaded data', style = 'color: #367AB4'),
                    tags$p('Dataset with', tags$span(textOutput('data_rows', inline = TRUE), style = 'color: red'), 'rows and', tags$span(textOutput('data_columns', inline = TRUE), style = 'color: red'), 'columns'),
                    dataTableOutput('show_data')
                  ),
                  conditionalPanel(
                    condition = 'input.switch_to_two > input.switch_to_one',
                    tags$h3('Loaded data - Household file', style = 'color: #367AB4'),
                    tags$p('Dataset with', tags$span(textOutput('data_rows_HH', inline = TRUE), style = 'color: red'), 'rows and', tags$span(textOutput('data_columns_HH', inline = TRUE), style = 'color: red'), 'columns'),
                    dataTableOutput('show_data_HH')
                  ),
                  conditionalPanel(
                    condition = 'input.switch_to_two > input.switch_to_one',
                    tags$h3('Loaded data - Individual file', style = 'color: #367AB4'),
                    tags$p('Dataset with', tags$span(textOutput('data_rows_I', inline = TRUE), style = 'color: red'), 'rows and', tags$span(textOutput('data_columns_I', inline = TRUE), style = 'color: red'), 'columns'),
                    dataTableOutput('show_data_I')
                  ),
                  tags$h3('Loaded totals', style = 'color: #367AB4'),
                  tags$p('Dataset with', tags$span(textOutput('totals_rows', inline = TRUE), style = 'color: red'), 'rows and', tags$span(textOutput('totals_columns', inline = TRUE), style = 'color: red'), 'columns'),
                  dataTableOutput('show_totals'),
                  tags$h3('Explore variables', style = 'color: #367AB4'),
                  selectInput('explore', NULL, NULL, width = '17%'),
                  conditionalPanel(
                    condition = 'input.explore',
                    checkboxInput('explore_as_cat', 'Explore as categorical')
                  ),
                  fluidRow(
                    column(6,
                           plotOutput('explore_plot')),
                    column(6,
                           tableOutput('explore_summary'))
                  )
        )
      )
    ),
    tabPanel(
      title = 'Calibration',
      sidebarLayout(
        sidebarPanel(width = 2,
                     conditionalPanel(
                       condition = 'input.stratification == true',
                       tags$h4('Choose strata', style = 'color: #367AB4'),
                       selectInput('strata', NULL, NULL, multiple = TRUE, selectize = FALSE, size = 12)
                     ),
                     actionButton('show_initial', 'Show with initial weights', class = 'btn btn-block'),
                     tags$h4('Method & Solver', style = 'color: #367AB4'),
                     fluidRow(
                       column(7,
                              radioButtons('method', 'Method', choices = c('Linear', 'Raking ratio', 'Logit', 'Linear bounded'), selected = 'Linear')),
                       column(5,
                              radioButtons('solver', 'Solver', choices = c('calib', 'nleqslv'), selected = 'calib'))
                     ),
                     conditionalPanel(
                       condition = 'input.method == "Logit" || input.method == "Linear bounded"',
                       fluidRow(
                         column(6,
                                numericInput('L', 'Lower bound', NULL, step = 0.01)),
                         column(6,
                                numericInput('U', 'Upper bound', NULL, step = 0.05))
                       )
                     ),
                     actionButton('calibrate', 'CALIBRATE', class = 'btn btn-danger btn-block', icon = icon('gear')),
                     tags$p(),
                     fluidRow(
                       column(6,
                              bookmarkButton('Bookmark', class = 'btn btn-block', icon = icon('bookmark-o'), title = "Bookmark Calif's state and get URL for ")),
                       column(6,
                              actionButton('save_result', 'Save', class = 'btn btn-primary btn-block', icon = icon('save')))
                     )
        ),
        mainPanel(width = 10,
                  uiOutput('ui_main')
        )
      )
    ),
    id = 'page', inverse = TRUE
  )
}

# function for pie chart with label inside, based on graphics::pie
new_pie <- function (x, labels = names(x), edges = 200, radius = 0.8, clockwise = FALSE, 
          init.angle = if (clockwise) 90 else 0, density = NULL, angle = 45, 
          col = NULL, border = NULL, lty = NULL, main = NULL, ...) 
{
  if (!is.numeric(x) || any(is.na(x) | x < 0)) 
    stop("'x' values must be positive.")
  if (is.null(labels)) 
    labels <- as.character(seq_along(x))
  else labels <- as.graphicsAnnot(labels)
  x <- c(0, cumsum(x)/sum(x))
  dx <- diff(x)
  nx <- length(dx)
  plot.new()
  pin <- par("pin")
  xlim <- ylim <- c(-1, 1)
  if (pin[1L] > pin[2L]) 
    xlim <- (pin[1L]/pin[2L]) * xlim
  else ylim <- (pin[2L]/pin[1L]) * ylim
  dev.hold()
  on.exit(dev.flush())
  plot.window(xlim, ylim, "", asp = 1)
  if (is.null(col)) 
    col <- if (is.null(density)) 
      c("white", "lightblue", "mistyrose", "lightcyan", 
        "lavender", "cornsilk")
  else par("fg")
  if (!is.null(col)) 
    col <- rep_len(col, nx)
  if (!is.null(border)) 
    border <- rep_len(border, nx)
  if (!is.null(lty)) 
    lty <- rep_len(lty, nx)
  angle <- rep(angle, nx)
  if (!is.null(density)) 
    density <- rep_len(density, nx)
  twopi <- if (clockwise) 
    -2 * pi
  else 2 * pi
  t2xy <- function(t) {
    t2p <- twopi * t + init.angle * pi/180
    list(x = radius * cos(t2p), y = radius * sin(t2p))
  }
  for (i in 1L:nx) {
    n <- max(2, floor(edges * dx[i]))
    P <- t2xy(seq.int(x[i], x[i + 1], length.out = n))
    polygon(c(P$x, 0), c(P$y, 0), density = density[i], 
            angle = angle[i], border = border[i], col = col[i], 
            lty = lty[i])
    P <- t2xy(mean(x[i + 0:1]))
    lab <- as.character(labels[i])
    if (!is.na(lab) && nzchar(lab)) {
      # changed part
      #lines(c(1, 1.05) * P$x, c(1, 1.05) * P$y)
      text(0.45 * P$x, 0.45 * P$y, labels[i], xpd = TRUE, 
           cex = 1.8, col = 'white', ...)
    }
  }
  title(main = main, ...)
  invisible(NULL)
}
environment(new_pie) <- getNamespace('graphics')

# calib function from sampling package with minor adjustments
calib <- function (Xs, d, total, q = rep(1, length(d)), method = c("linear", 
                                                          "raking", "truncated", "logit"), bounds = c(low = 0, upp = 10), 
          description = FALSE, max_iter = 500) 
{
  if (any(is.na(Xs)) | any(is.na(d)) | any(is.na(total)) | 
      any(is.na(q))) 
    stop("the input should not contain NAs")
  if (!(is.matrix(Xs) | is.array(Xs))) 
    Xs = as.matrix(Xs)
  if (is.matrix(Xs)) 
    if (length(total) != ncol(Xs)) 
      stop("Xs and total have different dimensions")
  if (is.vector(Xs) & length(total) != 1) 
    stop("Xs and total have different dimensions")
  if (any(is.infinite(q))) 
    stop("there are Inf values in the q vector")
  if (missing(method)) 
    stop("specify a method")
  if (!(method %in% c("linear", "raking", "logit", "truncated"))) 
    stop("the specified method is not in the list")
  if (method %in% c("linear", "raking") & !missing(bounds)) 
    stop("for the linear and raking the bounds are not allowed")
  EPS = .Machine$double.eps
  EPS1 = 1e-06
  n = length(d)
  lambda = as.matrix(rep(0, n))
  lambda1 = ginv(t(Xs * d * q) %*% Xs, tol = EPS) %*% (total - 
                                                         as.vector(t(d) %*% Xs))
  if (method == "linear" | max(abs(lambda1)) < EPS) 
    g = 1 + q * as.vector(Xs %*% lambda1)
  else if (method == "truncated") {
    if (!missing(bounds)) {
      # deleted part
      #if (bounds[2] <= 1 || bounds[1] >= 1 || bounds[1] > 
      #    bounds[2]) 
      #  warning("The conditions low<1<upp are not fulfilled")
    }
    else stop("Specify the bounds")
    g = 1 + q * as.vector(Xs %*% lambda1)
    list = 1:length(g)
    l = 0
    g1 = g
    Xs1 = Xs
    d1 = d
    t2 = total
    list1 = 1:length(g)
    q1 = q
    while (l < max_iter) {
      l = l + 1
      if (any(g < bounds[1]) | any(g > bounds[2])) {
        g[g < bounds[1]] = bounds[1]
        g[g > bounds[2]] = bounds[2]
        list = (1:length(g))[g > bounds[1] & g < bounds[2]]
        if (length(list) != 0) {
          g1 = g[list]
          t2 = total - as.vector(t(g[-list] * d[-list]) %*% 
                                   Xs[-list, ])
          Xs1 = Xs[list, ]
          d1 = d[list]
          q1 = q[list]
          list1 = list
        }
      }
      t1 = as.vector(t(d1) %*% Xs1)
      lambda1 = ginv(t(Xs1 * d1 * q1) %*% Xs1, tol = EPS) %*% 
        (t2 - t1)
      if (length(list1) > 1) 
        g1 = 1 + q1 * as.vector(Xs1 %*% lambda1)
      else if (length(list1) == 1) {
        g1 = 1 + q1 * as.vector(as.vector(Xs1) %*% as.vector(lambda1))
      }
      g[list1] = g1
      tr = crossprod(Xs, g * d)
      if (max(abs(tr - total)/total) < EPS1 & all(g >= 
                                                  bounds[1] & g <= bounds[2])) 
        break
    }
    if (l == max_iter) {
      # deleted part
      #cat("No convergence in", max_iter, "iterations with the given bounds. \n")
      #cat("The bounds for the g-weights are:", min(g), 
      #    " and ", max(g), "\n")
      #cat(" and the g-weights are given by g\n")
    }
  }
  else if (method == "raking") {
    lambda = as.matrix(rep(0, ncol(Xs)))
    w1 = as.vector(d * exp(Xs %*% lambda * q))
    for (l in 1:max_iter) {
      phi = t(Xs) %*% w1 - total
      T1 = t(Xs * w1)
      phiprim = T1 %*% Xs
      lambda = lambda - ginv(phiprim, tol = EPS) %*% phi
      w1 = as.vector(d * exp(Xs %*% lambda * q))
      if (any(is.na(w1)) | any(is.infinite(w1))) {
        # changed part
        #warning("No convergence")
        g = rep(1, length(d))
        break
      }
      tr = crossprod(Xs, w1)
      if (max(abs(tr - total)/total) < EPS1) 
        break
    }
    if (l == max_iter) {
      # changed part
      #warning("No convergence")
      g = rep(1, length(d))
    }
    else g = w1/d
  }
  else if (method == "logit") {
    if (bounds[2] <= 1 || bounds[1] >= 1 || bounds[1] > 
        bounds[2]) {}
    # deleted stop
      #stop("The conditions low<1<upp are not fulfilled")
    A = (bounds[2] - bounds[1])/((1 - bounds[1]) * (bounds[2] - 
                                                      1))
    u = rep(1, length(d))
    F = (bounds[1] * (bounds[2] - 1) + bounds[2] * (1 - 
                                                      bounds[1]) * u)/(bounds[2] - 1 + (1 - bounds[1]) * 
                                                                         u)
    w1 = as.vector(d * F)
    T = t(Xs * w1)
    phiprim = ginv(T %*% Xs, tol = EPS)
    g = F
    tr = crossprod(Xs, w1)
    if (max(abs(tr - total)/total) > EPS1 | any(g < bounds[1]) | 
        any(g > bounds[2])) {
      lambda1 = rep(0, ncol(Xs))
      list = 1:length(g)
      t2 = total
      Xs1 = Xs
      d1 = d
      g1 = g
      q1 = q
      list1 = 1:length(g)
      for (l in 1:max_iter) {
        if (any(g < bounds[1]) | any(g > bounds[2])) {
          g[g < bounds[1]] = bounds[1]
          g[g > bounds[2]] = bounds[2]
          list = (1:length(g))[g > bounds[1] & g < bounds[2]]
          if (length(list) != 0) {
            g1 = g[list]
            t2 = total - as.vector(t(g[-list] * d[-list]) %*% 
                                     Xs[-list, ])
            Xs1 = Xs[list, ]
            d1 = d[list]
            q1 = q[list]
            list1 = list
          }
        }
        t1 = as.vector(t(d1) %*% Xs1)
        phi = t(Xs1) %*% as.vector(d1 * g1) - t1
        T = t(Xs1 * as.vector(d1 * g1))
        phiprime = T %*% Xs1
        lambda1 = lambda1 - ginv(phiprime, tol = EPS) %*% 
          (as.vector(phi) - t2 + t1)
        u = exp(A * (Xs1 %*% lambda1 * q1))
        F = g1 = (bounds[1] * (bounds[2] - 1) + bounds[2] * 
                    (1 - bounds[1]) * u)/(bounds[2] - 1 + (1 - 
                                                             bounds[1]) * u)
        if (any(is.na(g1))) {
          # changed part
          #warning("no convergence")
          g1 = g = rep(1, length(d))
          break
        }
        g[list1] = g1
        tr = crossprod(Xs, g * d)
        if (max(abs(tr - total)/total) < EPS1 & all(g >= 
                                                    bounds[1] & g <= bounds[2])) 
          break
      }
      if (l == max_iter) {
        # deleted part
        #cat("no convergence in", max_iter, "iterations with the given bounds. \n")
        #cat("the bounds for the g-weights are:", min(g), 
        #    " and ", max(g), "\n")
        #cat(" and the g-weights are given by g\n")
        g = rep(1, length(d))
      }
    }
  }
  if (description && !is.null(g)) {
    par(mfrow = c(3, 2), pty = "s")
    hist(g)
    boxplot(g, main = "Boxplot of g")
    hist(d)
    boxplot(d, main = "Boxplot of d")
    hist(g * d)
    boxplot(g * d, main = "Boxplot of w=g*d")
    if (method %in% c("truncated", "raking", "logit")) 
      cat("number of iterations ", l, "\n")
    cat("summary - initial weigths d\n")
    print(summary(d))
    cat("summary - final weigths w=g*d\n")
    print(summary(as.vector(g * d)))
  }
  g
}
environment(calib) <- getNamespace('sampling')
assignInNamespace('calib', calib, 'sampling')

# inverse function of the first derivative of the distance function
F <- function(method, x, lambda, L, U) {
  if (method == 'Linear') return(1 + (x %*% lambda))
  if (method == 'Raking ratio') return(exp(x %*% lambda))
  if (method == 'Logit') {
    A <- (U - L) / ((1 - L) * (U - 1))
    u <- x %*% lambda
    B <- L * (U - 1) + U * (1 - L) * exp(A * u)
    C <- U - 1 + (1 - L) * exp(A * u)
    return(B / C)
  }
  if (method == 'Linear bounded') {
    u <- x %*% lambda
    if (u >= L - 1 && u <= U - 1) return(1 + u)
    return(Inf)
  }
}

# create 'x' matrix
make_x <- function(x, k, data, indices, miss) {
  if (x %in% k) {
    values <- setdiff(sort(unique(data[indices, x])), miss)
    return(do.call(cbind, lapply(values, function(z) as.numeric(data[indices, x] == z))))
  }
  return(as.matrix(data[indices, x], ncol = 1))
}

# calibration function
cal <- function(data, totals, stratification, miss, indicators, indicators_stats, num, ncat, nnum, k, n, method, solver, L, U, maxit, tol) {
  sums1 <- sums2 <- calib_settings <- rows <- NULL
  result <- numeric(nrow(data))
  current_strata <- 0
  if (stratification == TRUE) current_strata <- totals[, 1]
  for (j in current_strata) {
    indices <- which(data$STRAcalif == j)
    widthx <- 0
    if (stratification == TRUE) Xj <- totals[totals[, 1] == j, ][2:ncol(totals)]
    else Xj <- totals
    if (isTruthy(num)) if (0 %in% Xj[names(Xj) %in% num]) {
      showModal(modalDialog('There are zeros in numerical variables in the table of totals. Remove these numerical variables from the list of 
                            calibrated numerical variables in strata where this happens.', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    Xj <- Xj[which(!names(Xj) %in% indicators)]
    Xj1 <- Xj2 <- Xj
    Xj <- Xj[which(Xj > 0)]
    if (0 %in% apply(as.data.frame(data[indices, names(data) %in% num]), 2, sum))
      showModal(modalDialog('There are zero columns for some numerical variable in the data.', footer = modalButton('OK'), easyClose = TRUE))
    numsum <- NULL
    data_order <- NULL
    for (i in 1:(ncat + nnum)) {
      if (i %in% k) {
        values <- setdiff(sort(unique(data[indices, i])), miss)
        widthx <- widthx + length(values)
        data_order <- c(data_order, paste(names(data[i]), values, sep = '_'))
      }
      if (i %in% n) {
        widthx <- widthx + 1
        numsum <- c(numsum, widthx)
        data_order <- c(data_order, names(data[i]))
      }
    }
    if (setequal(data_order, names(Xj)) == FALSE) {
      # if more variables were set as auxiliary than the table of totals contains
      showModal(modalDialog('Calibration variables do not match to the table of totals. Please check the data and totals for possible error, or make sure that you have
                            specified auxiliary calibration variables correctly.', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    totals_order <- match(names(Xj), data_order)
    Xj <- Xj[data_order]
    Xj <- as.numeric(Xj)
    if (isTruthy(num)) {
      nr_places <- apply(as.matrix(data[indices, n]), 2, function(x) floor(log10(abs(mean(x)))))
      data[indices, n] <- t(t(data[indices, n]) / (10 ^ nr_places))
      Xj[numsum] <- t(t(Xj[numsum]) / (10 ^ nr_places))
    }
    x <- do.call(cbind, lapply(1:(ncat + nnum), function(i) make_x(i, k, data, indices, miss)))
    if (solver == 'initial') result[indices] <- data$WEIGHTcalif[indices]
    if (solver == 'nleqslv') {
      nr_lambda <- widthx
      d <- as.matrix(data$WEIGHTcalif[indices])
      nr_w <- length(d)
      equations <- function(z) {
        f <- sapply(1:nr_lambda, function(i) sum(sapply(1:nr_w, function(h) d[h] * F(method, x[h,], z, L, U) * x[h, i])) - Xj[i])
        f
      }
      p <- rep(0, nr_lambda)
      slv <- try(nleqslv(p, equations, control = list(maxit = maxit, xtol = tol)), silent = TRUE)
      if (class(slv) == 'try-error') result[indices] <- data$WEIGHTcalif[indices]
      else result[indices] <- d * F(method, x, slv$x, L, U)
      if (max(c(NA, NaN, Inf, -Inf) %in% result[indices]) == 1) result[indices] <- data$WEIGHTcalif[indices]
    }
    if (solver == 'calib') {
      d <- data[indices, 'WEIGHTcalif']
      if (method == 'Linear') g <- try(sampling::calib(Xs = x, d = d, total = Xj, method = 'linear', max_iter = maxit), silent = TRUE)
      if (method == 'Raking ratio') g <- try(sampling::calib(Xs = x, d = d, total = Xj, method = 'raking', max_iter = maxit), silent = TRUE)
      if (method == 'Logit') g <- try(sampling::calib(Xs = x, d = d, total = Xj, method = 'logit', bounds = c(low = L, upp = U), max_iter = maxit), silent = TRUE)
      if (method == 'Linear bounded') g <- try(sampling::calib(Xs = x, d = d, total = Xj, method = 'truncated', bounds = c(low = L, upp = U), max_iter = maxit), silent = TRUE)
      if (class(g) == 'try-error') result[indices] <- data$WEIGHTcalif[indices]
      else result[indices] <- g * d
      if (max(c(NA, NaN, Inf, -Inf) %in% result[indices]) == 1) result[indices] <- data$WEIGHTcalif[indices]
    }
    forcontrol1 <- result[indices] %*% x
    if (isTruthy(num)) {
      forcontrol1[numsum] <- forcontrol1[numsum] * (10 ^ nr_places)
      Xj[numsum] <- t(t(Xj[numsum]) * (10 ^ nr_places))
    }
    forcontrol2 <- paste0(sprintf("%.2f", forcontrol1 * 100 / Xj), '%')
    forcontrol1 <- forcontrol1[totals_order]
    forcontrol2 <- forcontrol2[totals_order]
    Xj1[Xj1 != 0] <- forcontrol1
    Xj2[Xj2 != 0] <- forcontrol2
    if (isTruthy(indicators) && isTruthy(indicators_stats)) {
      if ('Mean (value)' %in% indicators_stats) {
        Xj1 <- c(round(result[indices] %*% as.matrix(data[indices, indicators]) / sum(result[indices]), 2), Xj1)
        Xj2 <- c(round(result[indices] %*% as.matrix(data[indices, indicators]) / sum(result[indices]), 2), Xj2)
      }
      if ('Mean (%)' %in% indicators_stats) {
        if (stratification == TRUE) denom <- totals[totals[, 1] == j, indicators]
        if (stratification == FALSE) denom <- totals[indicators]
        Xj1 <- c(paste0(sprintf("%.2f", result[indices] %*% as.matrix(data[indices, indicators]) * 100 / sum(result[indices]) / denom), '%'), Xj1)
        Xj2 <- c(paste0(sprintf("%.2f", result[indices] %*% as.matrix(data[indices, indicators]) * 100 / sum(result[indices]) / denom), '%'), Xj2)
      }
      if ('Total (value)' %in% indicators_stats) {
        Xj1 <- c(round(result[indices] %*% as.matrix(data[indices, indicators]), 2), Xj1)
        Xj2 <- c(round(result[indices] %*% as.matrix(data[indices, indicators]), 2), Xj2)
      }
      if ('Total (%)' %in% indicators_stats) {
        if (stratification == TRUE) denom <- totals[totals[, 1] == j, indicators]
        if (stratification == FALSE) denom <- totals[indicators]
        Xj1 <- c(paste0(sprintf("%.2f", result[indices] %*% as.matrix(data[indices, indicators]) * 100 / denom), '%'), Xj1)
        Xj2 <- c(paste0(sprintf("%.2f", result[indices] %*% as.matrix(data[indices, indicators]) * 100 / denom), '%'), Xj2)
      }
    }
    names(Xj1) <- names(Xj2) <- ''
    rows <- c(rows, j)
    sums1 <- rbind(sums1, Xj1)
    sums2 <- rbind(sums2, Xj2)
    calib_settings <- rbind(calib_settings, c(stratum = j, solver = solver, method = method, lower_bound = L, upper_bound = U))
  }
  list(result = result, sums1 = sums1, sums2 = sums2, calib_settings = calib_settings, rows = rows)
}

server <- function(input, output, session)
{
  # reactive values
  values <- reactiveValues(wd = getwd(), data = NULL, data_HH = NULL, data_I = NULL, totals = NULL, calib_settings = NULL, weightsOUT = NULL, werecalibrated = NULL, 
                           out_totals1 = NULL, out_totals2 = NULL, out_stats = NULL, out_pie = NULL, out_hist = NULL, out_box = NULL, out_weights = NULL, out_hist_cal = NULL,
                           ui_main = NULL, out_text1 = 'calif_output', out_text2 = 'calif_settings', out_info = NULL, multistage = FALSE,
                           num = NULL, cat = NULL, select_num = NULL, select_weights = NULL, select_stra = NULL, select_cat = NULL, select_indicators = NULL, 
                           select_explore = NULL, select_ID_HH = NULL, select_ID_I = NULL, selected_num = NULL, selected_weights = '', selected_stra = '', 
                           selected_cat = NULL, selected_indicators = NULL, selected_ID_HH = NULL, selected_ID_I = NULL, update_strata = NULL)
  
  # to the data tab
  observeEvent(input$next_tab, {
    updateNavbarPage(session, 'page', 'Data')
  })
  
  # switch between type of calibration
  observeEvent(input$switch_to_one, {
    values$multistage <- FALSE
  })
  
  observeEvent(input$switch_to_two, {
    values$multistage <- TRUE
  })
  
  # to the calibration tab
  observeEvent(input$proceed, {
    updateNavbarPage(session, 'page', 'Calibration')
  })
  
  # working directory
  output$my_wd <- renderText({
    values$wd
  })
  
  # choose working directory
  observeEvent(input$change_wd, {
    path <- choose.dir(getwd())
    if (!is.na(path)) {
      setwd(path)
      values$wd <- path
    }
  })
  
  # data loading - one-stage calibration
  observe({
    req(input$load_data)
    if (!tools::file_ext(input$load_data$name) %in% c('txt', 'csv', 'sas7bdat')) {
      showModal(modalDialog('Incorrect format! Only .csv, .txt or .sas7bdat extensions are allowed.', footer = modalButton('OK'), easyClose = TRUE))
      return()
    }
    if (tools::file_ext(input$load_data$name) == 'sas7bdat') dat <- try(read_sas(input$load_data$datapath), silent = TRUE)
    if (tools::file_ext(input$load_data$name) != 'sas7bdat') dat <- try(read.table(input$load_data$datapath, sep = input$separator1, dec = input$decimal1, 
                                                                                   header = TRUE, stringsAsFactors = FALSE, check.names = FALSE), silent = TRUE)
    if (is.data.frame(dat)) {
      values$calib_settings <- NULL
      dat <- cbind(IDcalif = 1:nrow(dat), dat)
      values$data <- as.data.frame(dat)
      values$weightsOUT <- numeric(nrow(dat))
      values$werecalibrated <- NULL
      updateSelectInput(session, 'weights', choices = c(Choose = '', names(dat)[-1]))
      updateSelectInput(session, 'num', choices = names(dat)[-1])
      updateSelectInput(session, 'cat', choices = names(dat)[-1])
      updateSelectInput(session, 'indicators', choices = names(dat)[-1])
      updateSelectInput(session, 'stra', choices = c(Choose = '', names(dat)[-1]))
      updateSelectInput(session, 'explore', choices = c(Choose = '', names(dat)[-1]))
      updateSelectInput(session, 'strata', choices = character(0))
      values$select_weights <- c(Choose = '', names(dat)[-1])
      values$select_stra <- c(Choose = '', names(dat)[-1])
      values$select_num <- names(dat)[-1]
      values$select_cat <- names(dat)[-1]
      values$select_indicators <- names(dat)[-1]
      values$select_explore <- c(Choose = '', names(dat)[-1])
    }
    if (!is.data.frame(dat)) showModal(modalDialog('Data hasn\'t been loaded correctly!', footer = modalButton('OK'), easyClose = TRUE))
  })
  
  # data loading - two-stage calibration
  observe({
    req(input$load_data_HH)
    if (!tools::file_ext(input$load_data_HH$name) %in% c('txt', 'csv', 'sas7bdat')) {
      showModal(modalDialog('Incorrect format! Only .csv, .txt or .sas7bdat extensions are allowed.', footer = modalButton('OK'), easyClose = TRUE))
      return()
    }
    if (tools::file_ext(input$load_data_HH$name) == 'sas7bdat') dat <- try(read_sas(input$load_data_HH$datapath), silent = TRUE)
    if (tools::file_ext(input$load_data_HH$name) != 'sas7bdat') dat <- try(read.table(input$load_data_HH$datapath, sep = input$separator1, dec = input$decimal1, 
                                                                                   header = TRUE, stringsAsFactors = FALSE, check.names = FALSE), silent = TRUE)
    if (is.data.frame(dat)) {
      values$calib_settings <- NULL
      dat <- cbind(IDcalif = 1:nrow(dat), dat)
      values$data_HH <- as.data.frame(dat)
      values$weightsOUT <- numeric(nrow(dat))
      values$werecalibrated <- NULL
      updateSelectInput(session, 'weights', choices = c(Choose = '', names(dat)[-1]))
      updateSelectInput(session, 'stra', choices = c(Choose = '', names(dat)[-1]))
      updateSelectInput(session, 'strata', choices = character(0))
      updateSelectInput(session, 'ID_HH', choices = c(Choose = '', names(dat)[-1]))
      values$select_weights <- c(Choose = '', names(dat)[-1])
      values$select_stra <- c(Choose = '', names(dat)[-1])
      values$select_ID_HH <- c(Choose = '', names(dat)[-1])
    }
    if (!is.data.frame(dat)) showModal(modalDialog('Data hasn\'t been loaded correctly!', footer = modalButton('OK'), easyClose = TRUE))
  })
  
  observe({
    req(input$load_data_I)
    if (!tools::file_ext(input$load_data_I$name) %in% c('txt', 'csv', 'sas7bdat')) {
      showModal(modalDialog('Incorrect format! Only .csv, .txt or .sas7bdat extensions are allowed.', footer = modalButton('OK'), easyClose = TRUE))
      return()
    }
    if (tools::file_ext(input$load_data_I$name) == 'sas7bdat') dat <- try(read_sas(input$load_data_I$datapath), silent = TRUE)
    if (tools::file_ext(input$load_data_I$name) != 'sas7bdat') dat <- try(read.table(input$load_data_I$datapath, sep = input$separator3, dec = input$decimal3, 
                                                                                      header = TRUE, stringsAsFactors = FALSE, check.names = FALSE), silent = TRUE)
    if (is.data.frame(dat)) {
      values$data_I <- as.data.frame(dat)
      updateSelectInput(session, 'ID_I', choices = c(Choose = '', names(dat)))
      values$select_ID_I <- c(Choose = '', names(dat))
    }
    if (!is.data.frame(dat)) showModal(modalDialog('Data hasn\'t been loaded correctly!', footer = modalButton('OK'), easyClose = TRUE))
  })
  
  observe({
    req(input$load_data_HH, input$load_data_I, input$ID_HH, input$ID_I)
    req(input$ID_HH %in% names(values$data_HH), input$ID_I %in% names(values$data_I))
    if (max(duplicated(values$data_HH[, input$ID_HH])) > 0) {
      showModal(modalDialog('There are duplicates in Household file ID\'s!', footer = modalButton('OK'), easyClose = TRUE))
      req(FALSE)
    }
    if (!setequal(values$data_HH[, input$ID_HH], values$data_I[, input$ID_I])) {
      showModal(modalDialog('Household ID\'s in both files do not exactly match!', footer = modalButton('OK'), easyClose = TRUE))
      req(FALSE)
    }
    if (length(setdiff(intersect(names(values$data_HH), names(values$data_I)), input$ID_HH)) > 0) {
      showModal(modalDialog('There are some variables with the same names in both files. Each variable name (except for Household ID\'s) must be unique.
                            Please rename it, in order to avoid possible joining problems.', footer = modalButton('OK'), easyClose = TRUE))
      req(FALSE)
    }
    updateSelectInput(session, 'num', choices = list('Household file' = names(values$data_HH)[-1], 'Individual file' = names(values$data_I)))
    updateSelectInput(session, 'cat', choices = list('Household file' = names(values$data_HH)[-1], 'Individual file' = names(values$data_I)))
    updateSelectInput(session, 'indicators', choices = list('Household file' = names(values$data_HH)[-1], 'Individual file' = names(values$data_I)))
    updateSelectInput(session, 'explore', choices = c(Choose = '', list('Household file' = names(values$data_HH)[-1], 'Individual file' = names(values$data_I))))
    values$select_num <- list('Household file' = names(values$data_HH)[-1], 'Individual file' = names(values$data_I))
    values$select_cat <- list('Household file' = names(values$data_HH)[-1], 'Individual file' = names(values$data_I))
    values$select_indicators <- list('Household file' = names(values$data_HH)[-1], 'Individual file' = names(values$data_I))
    values$select_explore <- c(Choose = '', list('Household file' = names(values$data_HH)[-1], 'Individual file' = names(values$data_I)))
  })
  
  # totals loading
  observe({
    req(input$load_totals)
    if (!tools::file_ext(input$load_totals$name) %in% c('txt', 'csv', 'sas7bdat')) {
      showModal(modalDialog('Incorrect format! Only .csv, .txt or .sas7bdat extensions are allowed.', footer = modalButton('OK'), easyClose = TRUE))
      return()
    }
    if (tools::file_ext(input$load_totals$name) == 'sas7bdat') dat <- try(read_sas(input$load_totals$datapath), silent = TRUE)
    if (tools::file_ext(input$load_totals$name) != 'sas7bdat') dat <- try(read.table(input$load_totals$datapath, sep = input$separator2, dec = input$decimal2, 
                                                                                     header = TRUE, stringsAsFactors = FALSE, check.names = FALSE), silent = TRUE)
    if (is.data.frame(dat)) {
      if (nrow(dat) > 1 && ncol(dat) == 1) {
        showModal(modalDialog('Incorrect form of the Totals! Please check the decimal/separator
                              first.', footer = modalButton('OK'), easyClose = TRUE))
        return()
      }
      values$totals <- as.data.frame(dat)
    }
    if (!is.data.frame(dat)) showModal(modalDialog('Totals hasn\'t been loaded correctly!', footer = modalButton('OK'), easyClose = TRUE))
  })
  
  # show loaded data - one-stage calibration
  output$data_rows <- renderText({
    if (is.null(values$data)) return(0)
    nrow(values$data)
  })
  
  output$data_columns <- renderText({
    if (is.null(values$data)) return(0)
    ncol(values$data)
  })
  
  output$show_data <- renderDataTable({
    req(values$data)
    d <- values$data
    names(d)[1] <- 'Row'
    d
  }, options = list(pageLength = 5, lengthMenu = c(5, 10, 20, 50, 100, 500)))
  
  # show loaded data - two-stage calibration
  output$data_rows_HH <- renderText({
    if (is.null(values$data_HH)) return(0)
    nrow(values$data_HH)
  })
  
  output$data_rows_I <- renderText({
    if (is.null(values$data_I)) return(0)
    nrow(values$data_I)
  })
  
  output$data_columns_HH <- renderText({
    if (is.null(values$data_HH)) return(0)
    ncol(values$data_HH)
  })
  
  output$data_columns_I <- renderText({
    if (is.null(values$data_I)) return(0)
    ncol(values$data_I)
  })
  
  output$show_data_HH <- renderDataTable({
    req(values$data_HH)
    d <- values$data_HH
    names(d)[1] <- 'Row'
    d
  }, options = list(pageLength = 5, lengthMenu = c(5, 10, 20, 50, 100, 500)))
  
  output$show_data_I <- renderDataTable({
    values$data_I
  }, options = list(pageLength = 5, lengthMenu = c(5, 10, 20, 50, 100, 500)))
  
  # show loaded totals
  output$totals_rows <- renderText({
    if (is.null(values$totals)) return(0)
    nrow(values$totals)
  })
  
  output$totals_columns <- renderText({
    if (is.null(values$totals)) return(0)
    ncol(values$totals)
  })
  
  output$show_totals <- renderDataTable({
    values$totals
  }, options = list(pageLength = 5, lengthMenu = c(5, 10, 20, 50, 100, 500)))
  
  # explore variables
  output$explore_plot <- renderPlot({
    req(input$explore)
    if (isolate(!values$multistage)) data <- isolate(values$data)
    else if (input$explore %in% names(isolate(values$data_HH))) data <- isolate(values$data_HH)
    else if (input$explore %in% names(isolate(values$data_I))) data <- isolate(values$data_I)
    if (input$explore_as_cat == TRUE || is.character(data[, input$explore]))
      return(barplot(table(data[,input$explore]), border = FALSE, xlab = input$explore, ylab = 'Frequency', main = paste('Barplot of', input$explore, sep = ' ')))
    return(hist(data[,input$explore], col = 'grey', border = FALSE, xlab = input$explore, main = paste('Histogram of', input$explore, sep = ' ')))
  })
  
  output$explore_summary <- renderTable({
    req(input$explore)
    if (isolate(!values$multistage)) data <- isolate(values$data)
    else if (input$explore %in% names(isolate(values$data_HH))) data <- isolate(values$data_HH)
    else if (input$explore %in% names(isolate(values$data_I))) data <- isolate(values$data_I)
    if (input$explore_as_cat == TRUE || is.character(data[, input$explore])) {
      x <- as.data.frame(table(data[,input$explore], exclude = NULL))
      names(x) <- c('Value', 'Frequency')
      return(x)
    }
    x <- as.data.frame(as.array(summary(data[, input$explore])))
    names(x) <- rep('', 2)
    return(x)
  })
  
  # propose the most suitable solver for specific method
  observe({
    if (input$method == 'Logit') updateRadioButtons(session, 'solver', selected = 'nleqslv')
    if (input$method == 'Linear bounded') updateRadioButtons(session, 'solver', selected = 'calib')
  })
  
  # create values$num and values$cat in case of one-stage calibration
  observe({
    req(!values$multistage)
    values$num <- input$num
    values$cat <- input$cat
  })
  
  # data adjustment
  adj <- reactive({
    data <- values$data
    totals <- values$totals
    if (input$stratification == TRUE) choosecolumns <- c(which(names(data) %in% 'IDcalif'), which(names(data) %in% c(values$cat, values$num)), 
                                                         which(names(data) %in% input$indicators),
                                                         which(names(data) == input$stra), which(names(data) == input$weights))
    if (input$stratification == FALSE) {
      data <- cbind(data, STRAcalif = 0)
      choosecolumns <- c(which(names(data) %in% 'IDcalif'), which(names(data) %in% c(values$cat, values$num)), which(names(data) %in% input$indicators),
                         which(names(data) == 'STRAcalif'), which(names(data) == input$weights))
    }
    data <- data[, choosecolumns]
    if (input$stratification == TRUE) names(data)[ncol(data) - 1] <- 'STRAcalif'
    names(data)[ncol(data)] <- 'WEIGHTcalif'
    miss <- NULL
    if (isTruthy(input$miss)) miss <- unlist(strsplit(input$miss, ','))
    if (input$eliminate_miss == TRUE && !is.null(miss) && isTruthy(values$cat)) {
      for (i in miss) data[values$cat][data[values$cat] == i] <- NA
      data <- data[complete.cases(data),]
    }
    if (input$stratification == TRUE) {
      if (!isTruthy(intersect(data$STRAcalif, totals[, 1]))) {
        showModal(modalDialog('Strata in data and strata in totals are completely disjoint!', footer = actionButton('modal_ok', 'OK')))
        req(FALSE)
      }
      data <- data[data$STRAcalif %in% intersect(data$STRAcalif, totals[, 1]), ]
    }
    ncat <- length(values$cat)
    nnum <- length(values$num)
    k <- 0
    n <- 0
    if (isTruthy(values$cat)) k <- which(names(data)[-1] %in% values$cat)
    if (isTruthy(values$num)) n <- which(names(data)[-1] %in% values$num)
    totcol <- NULL
    for (i in 1:(ncat + nnum)) {
      if (i %in% k) {
        values <- setdiff(sort(unique(data[, i+1])), miss)
        totcol <- c(totcol, paste(names(data[i+1]), values, sep = '_'))
      }
      if (i %in% n) totcol <- c(totcol, names(data[i+1]))
    }
    if (!isTruthy(intersect(names(totals), totcol))) {
      showModal(modalDialog('Calibration variables you have selected are completely disjoint with the columns in the table of totals!', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    totcol <- c(totcol, input$indicators)
    if (input$stratification == TRUE) {
      totals <- totals[totals[, 1] %in% intersect(data$STRAcalif, totals[, 1]), c(1, which(names(totals) %in% totcol))]
      columns <- setdiff(names(totals), input$indicators)[-1]
      updateSelectInput(session, 'strata', choices = totals[, 1])
    }
    if (input$stratification == FALSE) {
      totals <- totals[names(totals) %in% totcol]
      columns <- setdiff(names(totals), input$indicators)
    }
    list(data = data, totals = totals, miss = miss, ncat = ncat, nnum = nnum, k = k, n = n, columns = columns)
  })
  
  # filtering selected strata
  val <- reactive({
    data <- adj()$data
    totals <- adj()$totals
    if (input$stratification == TRUE && isTruthy(input$strata)) {
      data <- adj()$data[adj()$data$STRAcalif %in% input$strata, ]
      totals <- adj()$totals[adj()$totals[, 1] %in% input$strata, ]
    }
    list(data = data, totals = totals)
  })
  
  # calculate average difference for linear solution
  AD_linear <- reactive({
    initial <- val()$data$WEIGHTcalif
    lin <- cal(val()$data[, -1], val()$totals, input$stratification, adj()$miss, input$indicators, input$indicators_stats, 
               values$num, adj()$ncat, adj()$nnum, adj()$k, adj()$n, 'Linear', 'calib', NULL, NULL, input$maxit, input$tol)$result
    mean(abs(lin - initial))
  })
  
  # check the inputs prior to calibration
  observeEvent(req(input$page == 'Calibration'), {
    if (!isTruthy(values$data) && !values$multistage) {
      showModal(modalDialog('Data are not loaded!', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    else if (!isTruthy(values$data_HH) && values$multistage) {
      showModal(modalDialog('Household file is not loaded!', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    else if (!isTruthy(values$data_I) && values$multistage) {
      showModal(modalDialog('Individual file is not loaded!', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    else if ((!isTruthy(input$ID_HH) || !isTruthy(input$ID_I)) && values$multistage) {
      showModal(modalDialog('Specify Household ID for both files!', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    else if (!isTruthy(values$totals)) {
      showModal(modalDialog('Totals are not loaded!', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    else if (!isTruthy(input$weights)) {
      showModal(modalDialog('Specify initial weights!', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    else if (!isTruthy(input$num) && !isTruthy(input$cat)) {
      showModal(modalDialog('Specify some auxiliary calibration variables!', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    else if (input$stratification == TRUE && !isTruthy(input$stra)) {
      showModal(modalDialog('Specify stratification variable!', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    else if (isTruthy(intersect(input$num, input$cat))) { 
      showModal(modalDialog('Variable(s) cannot be both numerical and categorical!', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    else if (isTruthy(intersect(input$indicators, c(input$num, input$cat)))) { 
      showModal(modalDialog('Indicator variables cannot be chosen from among variables for calibration.', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    else if (!isTruthy(input$stratification) && nrow(values$totals) > 1) {
      showModal(modalDialog('Table \'totals\' should consist of only 1 row in case of non-stratified calibration 
                            (with specified names in the header)', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    else if (!is.numeric(values$data[, input$weights]) && !values$multistage) {
      showModal(modalDialog('Weights are not in numeric form! You have probably set a wrong 
                            decimal point. Please check it.', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    else if (!is.numeric(values$data_HH[, input$weights]) && values$multistage) {
      showModal(modalDialog('Weights are not in numeric form! You have probably set a wrong 
                            decimal point. Please check it.', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    else if (input$stratification == TRUE && min(sapply(values$totals[-1], is.numeric)) == 0) {
      showModal(modalDialog('Some of the totals are not in numeric form! You have probably set a wrong 
                            decimal point. Please check it.', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    else if (input$stratification == FALSE && min(sapply(values$totals, is.numeric)) == 0) {
      showModal(modalDialog('Some of the totals are not in numeric form! You have probably set a wrong 
                            decimal point. Please check it.', footer = actionButton('modal_ok', 'OK')))
      req(FALSE)
    }
    else if (isTruthy(input$num)) {
      if (!values$multistage) {
        if (min(sapply(values$data[input$num], is.numeric)) == 0) {
          showModal(modalDialog('Numerical variables are not in numeric form! You have probably set a wrong 
                            decimal point. Please check it.', footer = actionButton('modal_ok', 'OK')))
          req(FALSE)
        }
      }
      if (values$multistage) {
        if (length(intersect(names(values$data_HH), input$num)) > 0) if (min(sapply(values$data_HH[names(values$data_HH) %in% input$num], is.numeric)) == 0) {
          showModal(modalDialog('Numerical variables are not in numeric form! You have probably set a wrong 
                                decimal point. Please check it.', footer = actionButton('modal_ok', 'OK')))
          req(FALSE)
        }
        if (length(intersect(names(values$data_I), input$num)) > 0) if (min(sapply(values$data_I[names(values$data_I) %in% input$num], is.numeric)) == 0) {
          showModal(modalDialog('Numerical variables are not in numeric form! You have probably set a wrong 
                                decimal point. Please check it.', footer = actionButton('modal_ok', 'OK')))
          req(FALSE)
        }
      }
    }
    
    if (values$multistage) {
      num_HH <- intersect(names(values$data_HH), input$num)
      cat_HH <- intersect(names(values$data_HH), input$cat)
      indicators_HH <- intersect(names(values$data_HH), input$indicators)
      num_I <- intersect(names(values$data_I), input$num)
      cat_I <- intersect(names(values$data_I), input$cat)
      indicators_I <- intersect(names(values$data_I), input$indicators)
      values$cat <- cat_HH
      values$num <- c(num_HH, num_I)
      
      miss <- NULL
      if (isTruthy(input$miss)) miss <- unlist(strsplit(input$miss, ','))
      dat1 <- values$data_HH[names(values$data_HH) %in% c('IDcalif', input$ID_HH, input$weights, input$stra, cat_HH, num_HH, indicators_HH)]
      dat2 <- values$data_I[names(values$data_I) %in% c(input$ID_I, num_I, indicators_I)]
      if (isTruthy(cat_I)) {
        for (x in cat_I) {
          v <- setdiff(sort(unique(values$data_I[, x])), miss)
          dat3 <- as.data.frame(do.call(cbind, lapply(v, function(z) as.numeric(values$data_I[, x] == z))))
          names(dat3) <- paste(x, v, sep = '_')
          dat2 <- cbind(dat2, dat3)
          values$num <- c(values$num, names(dat3))
        }
      }
      dat2 <- aggregate(dat2[names(dat2) %in% c(values$num, indicators_I)], dat2[input$ID_I], FUN = sum)
      values$data <- merge(dat1, dat2, by.x = input$ID_HH, by.y = input$ID_I, all = FALSE, sort = FALSE)
      if (!identical(values$data[input$ID_HH], values$data_HH[input$ID_HH])) {
        showModal(modalDialog('Joining error! Check the joining ID\'s.', footer = actionButton('modal_ok', 'OK')))
        req(FALSE)
      }
      showModal(modalDialog('Joined file has been created.', footer = modalButton('OK'), easyClose = TRUE))
    }
    
    adj()  # strata list is set up
    
    updateSelectInput(session, 'strata', selected = values$update_strata)  # re-set selected strata from previous task
  })
  
  observeEvent(input$modal_ok, {
    updateNavbarPage(session, 'page', 'Data')
    removeModal()
  })
  
  # CALIBRATE button
  observeEvent(input$calibrate, {
    showModal(modalDialog('Please wait...', footer = NULL))
    
    # creating output UI first
    values$ui_main <- tagList(
      fluidRow(
        column(4,
               conditionalPanel(
                 condition = 'output.result_stats',
                 tags$h4('Results', style = 'color: #367AB4; margin-left: 0.67em'),
                 tags$br()
               ),
               tableOutput('result_stats'),
               helpText(textOutput('hover_info'), style = 'text-align: justify')),
        column(2,
               conditionalPanel(
                 condition = 'output.result_pie',
                 tags$h4('Average difference feasibility', style = 'text-align: center; color: #367AB4')
               ),
               plotOutput('result_pie', hover = hoverOpts('hover_pie_plot', delay = 0), height = '260px')),
        column(3,
               conditionalPanel(
                 condition = 'output.result_hist',
                 tags$h4('Histogram of quotients', style = 'text-align: center; color: #367AB4')
               ),
               plotOutput('result_hist', hover = hoverOpts('hover_hist_plot', delay = 0), height = '300px')),
        column(3,
               conditionalPanel(
                 condition = 'output.result_box',
                 tags$h4('Histogram of calibration weights', style = 'text-align: center; color: #367AB4')
               ),
               plotOutput('result_hist_cal', height = '300px'))  
      ),
      conditionalPanel(
        condition = 'output.result_totals',
        tags$h4('Totals obtained', style = 'color: #367AB4'),
        checkboxInput('show_as_values', 'Show obtained totals as values')
      ),
      tableOutput('result_totals'),
      conditionalPanel(
        condition = 'output.result_weights',
        tags$br(),
        tags$br(),
        fluidRow(
          column(4,
                 tags$h4('Weights & Quotients', style = 'color: #367AB4')),
          column(4, offset = 1,
                 tags$h4('Boxplots of weights', style = 'text-align: center; color: #367AB4'))
        )
      ),
      fluidRow(
        column(4,
               dataTableOutput('result_weights')),
        column(4, offset = 1,
               plotOutput('result_box', height = '500px'))
      )
    )
    
    if (input$method %in% c('Logit', 'Linear bounded')) {
      if (!isTruthy(input$L) || !isTruthy(input$U)) {
        showModal(modalDialog('Specify lower and upper bound, ideally L < 1 < U!', footer = modalButton('OK'), easyClose = TRUE))
        req(FALSE)
      }
      if (input$L <= 1 && input$U <= 1 && (input$method != 'Linear bounded' || input$solver != 'calib')) {
        showModal(modalDialog('Combination of Lower bound <= 1 as well as Upper bound <= 1 can be used only with Linear bounded method and calib as a solver.', 
                              footer = modalButton('OK'), easyClose = TRUE))
        req(FALSE)
      }
      if (input$L >= 1 && input$U >= 1 && (input$method != 'Linear bounded' || input$solver != 'calib')) {
        showModal(modalDialog('Combination of Lower bound >= 1 as well as Upper bound >= 1 can be used only with Linear bounded method and calib as a solver.', 
                              footer = modalButton('OK'), easyClose = TRUE))
        req(FALSE)
      }
      if ((input$L == 1 || input$U == 1) && (input$method != 'Linear bounded' || input$solver != 'calib')) {
        showModal(modalDialog('Some of the bounds equal to 1 can be used only with Linear bounded method and calib as a solver.', footer = modalButton('OK'), easyClose = TRUE))
        req(FALSE)
      }
      if (input$L == input$U) {
        showModal(modalDialog('Calibration with equal bounds does not make sense.', footer = modalButton('OK'), easyClose = TRUE))
        req(FALSE)
      }
      if (input$L > input$U) {
        showModal(modalDialog('Lower bound must be smaller than upper bound, ideally L < 1 < U.', footer = modalButton('OK'), easyClose = TRUE))
        req(FALSE)
      }
    }
    if (input$stratification == TRUE && !isTruthy(input$strata)) {
      showModal(modalDialog('Choose some strata!', footer = modalButton('OK'), easyClose = TRUE))
      req(FALSE)
    }
    columns <- adj()$columns
    if (isTruthy(input$indicators) && isTruthy(input$indicators_stats)) 
      columns <- c(apply(expand.grid(input$indicators, input$indicators_stats, stringsAsFactors = FALSE), 1, paste, collapse = '.'), columns)
    res <- cal(val()$data[, -1], val()$totals, input$stratification, adj()$miss, input$indicators, input$indicators_stats, 
               values$num, adj()$ncat, adj()$nnum, adj()$k, adj()$n, input$method, input$solver, input$L, input$U, input$maxit, input$tol)
    removeModal()
    colnames(res$sums1) <- colnames(res$sums2) <- columns
    if (input$stratification == TRUE) {
      res$sums1 <- cbind(Stratum = res$rows, res$sums1)
      res$sums2 <- cbind(Stratum = res$rows, res$sums2)
    }
    initial <- val()$data$WEIGHTcalif
    arecalibrated <- val()$data$IDcalif
    values$werecalibrated <- sort(unique(c(values$werecalibrated, arecalibrated)))
    werenotcalibrated <- setdiff(values$data$IDcalif, values$werecalibrated)
    values$calib_settings <- rbind(values$calib_settings, res$calib_settings)
    values$weightsOUT[arecalibrated] <- res$result
    values$weightsOUT[werenotcalibrated] <- values$data[werenotcalibrated, input$weights]
    ratios <- res$result / initial
    L_w1 <- round(1 / min(initial), 3) + 0.001
    if (min(res$result) < 1) {
      L_w1 <- round(1 / min(initial[res$result < 1]), 3) + 0.001
      showNotification('Some of the calibration weights are less than 1! Use the Minimum realistic lower bound to get all weights greater or equal than 1.', 
                       duration = NULL, id = 'notification', type = 'error')
      updateNumericInput(session, 'L', value = L_w1)
    }
    if (min(res$result) >= 1) removeNotification('notification')
    values$out_stats <- data.frame(c('Initial weights interval', 'Calibration weights interval', 'Lower bound obtained', 'Upper bound obtained', 
                                     'Average weight quotient', 'Average difference', 'SD of cal. and initial weights', 'Minimum realistic lower bound'),
                                   c(min(initial), min(res$result), min(ratios), max(ratios), mean(ratios), mean(abs(res$result - initial)), sd(res$result), L_w1),
                                   c(max(initial), max(res$result), rep(NA, 4), sd(initial), NA))
    values$out_pie <- AD_linear() / mean(abs(res$result - initial))
    values$out_hist <- ratios
    values$out_hist_cal <- res$result
    values$out_totals1 <- res$sums1
    values$out_totals2 <- res$sums2
    values$out_weights <- data.frame(Row = arecalibrated, Initial = round(initial, 5), Calibration = round(res$result, 5), Quotients = round(ratios, 5))
    values$out_box <- list(initial, res$result)
  })
  
  # show with initial weights
  observeEvent(input$show_initial, {
    # creating output UI first
    values$ui_main <- tagList(
      tags$h4('Totals obtained', style = 'color: #367AB4'),
      checkboxInput('show_as_values', 'Show obtained totals as values'),
      tableOutput('result_totals')
    )
    
    if (input$stratification == TRUE && !isTruthy(input$strata)) {
      showModal(modalDialog('Choose some strata!', footer = modalButton('OK'), easyClose = TRUE))
      req(FALSE)
    }
    values$out_stats <- values$out_pie <- values$out_hist <- values$out_box <- values$out_weights <- values$out_hist_cal <- NULL
    columns <- adj()$columns
    if (isTruthy(input$indicators) && isTruthy(input$indicators_stats)) 
      columns <- c(apply(expand.grid(input$indicators, input$indicators_stats, stringsAsFactors = FALSE), 1, paste, collapse = '.'), columns)
    res <- cal(val()$data[, -1], val()$totals, input$stratification, adj()$miss, input$indicators, input$indicators_stats, 
               values$num, adj()$ncat, adj()$nnum, adj()$k, adj()$n, input$method, 'initial', NULL, NULL, input$maxit, input$tol)
    colnames(res$sums1) <- colnames(res$sums2) <- columns
    if (input$stratification == TRUE) {
      res$sums1 <- cbind(Stratum = res$rows, res$sums1)
      res$sums2 <- cbind(Stratum = res$rows, res$sums2)
    }
    values$out_totals1 <- res$sums1
    values$out_totals2 <- res$sums2
  })
  
  # RENDERING OUTPUTS
  
  # UI for mainPanel of Calibration tab
  output$ui_main <- renderUI({
    values$ui_main
  })
  
  output$result_totals <- renderTable({
    if (input$show_as_values == TRUE) values$out_totals1
    else values$out_totals2
  }, align = 'r')
  
  output$result_stats <- renderTable({
    values$out_stats
  }, colnames = FALSE, na = '', digits = 3)
  
  output$result_pie <- renderPlot({
    req(!values$out_pie %in% c(NaN, NA, Inf, -Inf))
    par(mar = rep(0, 4))
    new_pie(c(values$out_pie, max(0, 1 - values$out_pie)), labels = paste0(sprintf("%.1f", values$out_pie * 100), '%'), col = c('#367AB4', 'white'), 
            clockwise = TRUE, init.angle = 90, border = '#367AB4')
  })
  
  output$result_hist <- renderPlot({
    req(values$out_hist)
    par(mar = c(3,4,2,2))
    hist(values$out_hist, col = '#5AC5CB', border = FALSE, main = NULL, xlab = NULL, ylab = NULL, breaks = 20)
  })
  
  output$result_hist_cal <- renderPlot({
    req(values$out_hist_cal)
    par(mar = c(3,2,2,2))
    hist(values$out_hist_cal, col = '#D9534F', border = FALSE, main = NULL, xlab = NULL, ylab = NULL, breaks = 20)
  })
  
  output$hover_info <- renderText({
    if (!is.null(input$hover_pie_plot))
      return('Linear method (or sometimes Raking Ratio) introduces the lowest average difference for each calibration task. 
             The piechart reflects the quotient of the average differences of linear and current method. 
             It should not be significantly less than 100%, especially for bounded calibrations.')
    if (!is.null(input$hover_hist_plot))
      return('Displays the distribution of quotients \'calibration weight/initial weight\'.
             It should resemble normal distribution as much as possible.')
  })
  
  output$result_weights <- renderDataTable({
    values$out_weights
  }, options = list(pageLength = 20, lengthMenu = c(20, 50, 100, 500), searching = FALSE))
  
  output$result_box <- renderPlot({
    req(values$out_box)
    par(bty = 'n')
    boxplot(values$out_box[[1]], values$out_box[[2]], names = NULL, col = c('#D0EDF3', '#D9534F'), xaxt = 'n')
    axis(1, at = 1:2, labels = c('initial', 'calibration'), tick = FALSE, line = -0.5, cex.axis = 1.2)
    abline(h = 1, lty = 2, col = 'lightgrey')
  })
  
  # save results - show modal
  observeEvent(input$save_result, {
    showModal(modalDialog(
      textInput('save_folder', 'Output folder - your Working directory', value = values$wd, width = '100%'),
      textInput('save_data', 'Output data', value = values$out_text1, width = '100%'),
      textInput('save_settings', 'Calibration settings', value = values$out_text2, width = '100%'),
      checkboxInput('save_overwrite', 'Overwrite existing outputs', value = TRUE),
      tags$em(textOutput('save_info')),
      tags$br(),
      fluidRow(
        column(12, align = 'center',
               actionButton('save_it', 'Save', icon = icon('save'), width = '18%', class = 'btn btn-primary'))
      ),
      title = 'Save results',
      footer = NULL,
      easyClose = TRUE
    ))
    values$out_info <- NULL
  })
  
  # save results
  observeEvent(input$save_it, {
    values$out_text1 <- input$save_data
    values$out_text2 <- input$save_settings
    if (file.exists(input$save_folder)) {
      setwd(input$save_folder)
      values$wd <- input$save_folder
    }
    else {
      values$out_info <- 'Output folder does not exist!'
      req(FALSE)
    }
    if (input$save_overwrite == FALSE) {
      if (file.exists(paste0(file.path(input$save_folder, input$save_data), '.csv')) || file.exists(paste0(file.path(input$save_folder, input$save_settings), '.csv'))) {
        values$out_info <- 'The files already exist and are not allowed to be overwritten!'
        req(FALSE)
      }
    }
    if (input$save_data == input$save_settings) {
      values$out_info <- 'The files cannot have the same name!'
      req(FALSE)
    }
    settings <- values$calib_settings
    if (!is.null(settings)) {
      settings <- as.data.frame(settings)
      settings[settings$method %in% c('Linear', 'Raking ratio'), c('lower_bound', 'upper_bound')] <- NA
      settings <- settings[!duplicated(settings$stratum, fromLast = TRUE), ]
      settings <- settings[order(settings$stratum), ]
    }
    if (!values$multistage) {
      data <- values$data
      data <- cbind(data, CalibrationWeight = values$weightsOUT)
      w1 <- try(write.table(data[, -1], paste0(file.path(input$save_folder, input$save_data), '.csv'), 
                            sep = input$separator1, dec = input$decimal1, na = '', row.names = FALSE), silent = TRUE)
      w2 <- try(write.table(settings, paste0(file.path(input$save_folder, input$save_settings), '.csv'), 
                            sep = input$separator1, dec = input$decimal1, na = '', row.names = FALSE), silent = TRUE)
      if (class(w1) == 'try-error' || class(w2) == 'try-error') {
        values$out_info <- 'Cannot write to file(s)!'
        req(FALSE)
      }
    }
    if (values$multistage) {
      data_HH <- cbind(values$data_HH, CalibrationWeight = values$weightsOUT)
      data_I <- merge(values$data_I, data_HH[c(input$ID_HH, 'CalibrationWeight')], by.x = input$ID_I, by.y = input$ID_HH, sort = FALSE)
      w1 <- try(write.table(data_HH[, !names(data_HH) %in% 'IDcalif'], paste0(file.path(input$save_folder, input$save_data), '_HH.csv'), 
                            sep = input$separator1, dec = input$decimal1, na = '', row.names = FALSE), silent = TRUE)
      w2 <- try(write.table(data_I, paste0(file.path(input$save_folder, input$save_data), '_IND.csv'), 
                            sep = input$separator3, dec = input$decimal3, na = '', row.names = FALSE), silent = TRUE)
      w3 <- try(write.table(settings, paste0(file.path(input$save_folder, input$save_settings), '.csv'), 
                            sep = input$separator1, dec = input$decimal1, na = '', row.names = FALSE), silent = TRUE)
      if (class(w1) == 'try-error' || class(w2) == 'try-error' || class(w3) == 'try-error') {
        values$out_info <- 'Cannot write to file(s)!'
        req(FALSE)
      }
    }
    values$out_info <- 'The files have been saved.'
  })
  
  # show save info
  output$save_info <- renderText({
    values$out_info
  })
  
  # save selected strata in order to update it when toggling tabs
  observe({
    values$update_strata <- input$strata
  })
  
  # save selected values in order to bookmark them later
  observe({
    values$selected_weights <- input$weights
    values$selected_stra <- input$stra
    values$selected_num <- input$num
    values$selected_cat <- input$cat
    values$selected_indicators <- input$indicators
    values$selected_ID_HH <- input$ID_HH
    values$selected_ID_I <- input$ID_I
  })
  
  # bookmarking
  onBookmark(function(state) {
    state$values$data <- values$data
    state$values$data_HH <- values$data_HH
    state$values$data_I <- values$data_I
    state$values$totals <- values$totals
    state$values$multistage <- values$multistage
    state$values$num <- values$num
    state$values$cat <- values$cat
    state$values$select_weights <- values$select_weights
    state$values$select_stra <- values$select_stra
    state$values$select_num <- values$select_num
    state$values$select_cat <- values$select_cat
    state$values$select_indicators <- values$select_indicators
    state$values$select_explore <- values$select_explore
    state$values$select_ID_HH <- values$select_ID_HH
    state$values$select_ID_I <- values$select_ID_I
    state$values$selected_weights <- values$selected_weights
    state$values$selected_stra <- values$selected_stra
    state$values$selected_num <- values$selected_num
    state$values$selected_cat <- values$selected_cat
    state$values$selected_indicators <- values$selected_indicators
    state$values$selected_ID_HH <- values$selected_ID_HH
    state$values$selected_ID_I <- values$selected_ID_I
    state$values$wd <- values$wd
    state$values$calib_settings <- values$calib_settings
    state$values$weightsOUT <- values$weightsOUT
    state$values$werecalibrated <- values$werecalibrated
    state$values$out_totals1 <- values$out_totals1
    state$values$out_totals2 <- values$out_totals2
    state$values$out_stats <- values$out_stats
    state$values$out_pie <- values$out_pie
    state$values$out_hist <- values$out_hist
    state$values$out_box <- values$out_box
    state$values$out_weights <- values$out_weights
    state$values$ui_main <- values$ui_main
    state$values$out_text1 <- values$out_text1
    state$values$out_text2 <- values$out_text2
    state$values$out_info <- values$out_info
  })
  
  onRestore(function(state) {
    values$data <- state$values$data
    values$data_HH <- state$values$data_HH
    values$data_I <- state$values$data_I
    values$totals <- state$values$totals
    values$multistage <- state$values$multistage
    values$num <- state$values$num
    values$cat <- state$values$cat
    values$select_weights <- state$values$select_weights
    values$select_stra <- state$values$select_stra
    values$select_num <- state$values$select_num
    values$select_cat <- state$values$select_cat
    values$select_indicators <- state$values$select_indicators
    values$select_explore <- state$values$select_explore
    values$select_ID_HH <- state$values$select_ID_HH
    values$select_ID_I <- state$values$select_ID_I
    values$selected_weights <- state$values$selected_weights
    values$selected_stra <- state$values$selected_stra
    values$selected_num <- state$values$selected_num
    values$selected_cat <- state$values$selected_cat
    values$selected_indicators <- state$values$selected_indicators
    values$selected_ID_HH <- state$values$selected_ID_HH
    values$selected_ID_I <- state$values$selected_ID_I
    updateSelectInput(session, 'weights', choices = state$values$select_weights, selected = state$values$selected_weights)
    updateSelectInput(session, 'stra', choices = state$values$select_stra, selected = state$values$selected_stra)
    updateSelectInput(session, 'num', choices = state$values$select_num, selected = state$values$selected_num)
    updateSelectInput(session, 'cat', choices = state$values$select_cat, selected = state$values$selected_cat)
    updateSelectInput(session, 'indicators', choices = state$values$select_indicators, selected = state$values$selected_indicators)
    updateSelectInput(session, 'explore', choices = state$values$select_explore)
    updateSelectInput(session, 'ID_HH', choices = state$values$select_ID_HH, selected = state$values$selected_ID_HH)
    updateSelectInput(session, 'ID_I', choices = state$values$select_ID_I, selected = state$values$selected_ID_I)
    values$wd <- state$values$wd
    values$calib_settings <- state$values$calib_settings
    values$weightsOUT <- state$values$weightsOUT
    values$werecalibrated <- state$values$werecalibrated
    values$out_totals1 <- state$values$out_totals1
    values$out_totals2 <- state$values$out_totals2
    values$out_stats <- state$values$out_stats
    values$out_pie <- state$values$out_pie
    values$out_hist <- state$values$out_hist
    values$out_box <- state$values$out_box
    values$out_weights <- state$values$out_weights
    values$ui_main <- state$values$ui_main
    values$out_text1 <- state$values$out_text1
    values$out_text2 <- state$values$out_text2
    values$out_info <- state$values$out_info
  })
  
  setBookmarkExclude(c('load_data', 'load_data_HH', 'load_data_I', 'load_totals', 'page', 'calibrate', 'change_wd', 'show_initial', 'next_tab', 'proceed', 'save_result', 'modal_ok', 'save_it'))
}

shinyApp(ui, server, enableBookmarking = 'server', options = list(launch.browser = TRUE))
}

















